@@ -13,11 +13,28 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the Free Software           *
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307   *
+ *   USA.                                                                  *
+ *                                                                         *
+ *   For license exceptions see LICENSE.EXC file, attached with the source *
+ *   code.                                                                 *
+ *                                                                         *
  ***************************************************************************/
 
 #ifndef __PROGRESSDIALOG_H__
 #define __PROGRESSDIALOG_H__
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 // QT Includes
 #include <qprogressdialog.h>
 #include <qlayout.h>
@@ -33,7 +50,7 @@ class ProgressDialog : public QProgressDialog
         ProgressDialog( QWidget* parent = 0, bool p = false, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
         ~ProgressDialog();
 	
-	void print(QString in, QString out="");
+	void print(const QString& in, const QString& out="") ;
 	void inc();
 	void init( int numOfFiles );
 	
