@@ -14,11 +14,29 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the Free Software           *
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307   *
+ *   USA.                                                                  *
+ *                                                                         *
+ *   For license exceptions see LICENSE.EXC file, attached with the source *
+ *   code.                                                                 *
+ *                                                                         *
  ***************************************************************************/
 
 #ifndef _IMAGEFILEINFO_H_
 #define _IMAGEFILEINFO_H_
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// QT
 #include <qstring.h>
 
 #define IMAGE	0
@@ -29,7 +47,7 @@ class QString;
 class ImageFileInfo
 {
 public:
-	ImageFileInfo(QString filename, int type, bool r=true);
+	ImageFileInfo(const QString& filename, int type, bool r=true);
 	~ImageFileInfo();
 	
 	bool hasInfo();
@@ -43,27 +61,27 @@ public:
 		QString shortDescription, QString longDescription,
 		QString fileDest=QString());
 	
-	void update(QString destDir);
+	void update(const QString& destDir);
 	
 	QString getTitle() const;
-	void setTitle(QString title);
+	void setTitle(const QString& title);
 	QString getEvent() const;
-	void setEvent(QString event);
+	void setEvent(const QString& event);
 	QString getLocation() const;
-	void setLocation(QString location);
+	void setLocation(const QString& location);
 	QString getPeople() const;
-	void setPeople(QString people);
+	void setPeople(const QString& people);
 	QString getDate() const;
-	void setDate(QString date);
+	void setDate(const QString& date);
 	QString getDescription() const;
-	void setDescription(QString description);
+	void setDescription(const QString& description);
 	QString getShortDescription() const;
-	void setShortDescription(QString shortdesc);
+	void setShortDescription(const QString& shortdesc);
 	QString getLongDescription() const;
-	void setLongDescription(QString longdesc);
+	void setLongDescription(const QString& longdesc);
 	
 protected:
-	bool verif(QString info);
+	bool verif(const QString& info);
 	void read(bool r=true);
 	int type;
 	
