@@ -1,7 +1,7 @@
 /*
  * qxcfi.cpp: A Qt 3 plug-in for reading GIMP XCF image files
  * Copyright (C) 2001 lignum Computing, Inc. <allen@lignumcomputing.com>
- * $Id: qxcfi.cpp,v 1.1 2003/12/04 16:16:01 groult Exp $
+ * $Id: qxcfi.cpp,v 1.4 2004/11/22 14:01:36 groult Exp $
  *
  * This plug-in is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,298 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
+#include <kdebug.h>
 
-//#include <qimageformatplugin.h>
-#include <qimage.h>
 #include <qiodevice.h>
 #include <stdlib.h>
-#include "qvaluestack.h"
-#include "qvaluevector.h"
-
-#include "gimp.h"
-
-// From Qt3 QTL, needed for QValueVector and QValueStack
-/*
-template <class BiIterator, class BiOutputIterator>
-inline BiOutputIterator qCopyBackward( BiIterator _begin, BiIterator _end,
-				       BiOutputIterator _dest )
-{
-    while ( _begin != _end )
-	*--_dest = *--_end;
-    return _dest;
-}
-*/
-
-/*
-template <class ForwardIterator, class T>
-inline void qFill( ForwardIterator first, ForwardIterator last, const T& val )
-{
-    for ( ; first != last; ++first )
-	*first = val;
-}
-// QTL done
-*/
+#include "qxcfi.h"
 
 const float INCHESPERMETER = (100. / 2.54);
 
-//! Plug-in for loading a GIMP XCF image file directly.
-/*!
- * This class uses the Qt 3.0 Image format plug-in loader to provide
- * the ability to read The GIMP XCF image files. This plug-in will
- * be dynamically loaded as needed.
- */
-class XCFImageFormat{
-
-  /*!
-   * Each layer in an XCF file is stored as a matrix of
-   * 64-pixel by 64-pixel images. The GIMP has a sophisticated
-   * method of handling very large images as well as implementing
-   * parallel processing on a tile-by-tile basis. Here, though,
-   * we just read them in en-masse and store them in a matrix.
-   */
-  typedef QValueVector< QValueVector< QImage > > Tiles;
-
-  /*!
-   * Each GIMP image is composed of one or more layers. A layer can
-   * be one of any three basic types: RGB, grayscale or indexed. With an
-   * optional alpha channel, there are six possible types altogether.
-   *
-   * Note: there is only ever one instance of this structure. The
-   * layer info is discarded after it is merged into the final QImage.
-   */
-  struct Layer {
-    Q_UINT32 width;		//!< Width of the layer
-    Q_UINT32 height;		//!< Height of the layer
-    Q_INT32 type;		//!< Type of the layer (GimpImageType)
-    char* name;			//!< Name of the layer
-    Q_UINT32 hierarchy_offset;	//!< File position of Tile hierarchy
-    Q_UINT32 mask_offset;	//!< File position of mask image
-
-    uint nrows;			//!< Number of rows of tiles (y direction)
-    uint ncols;			//!< Number of columns of tiles (x direction)
-
-    Tiles image_tiles;		//!< The basic image
-    //! For Grayscale and Indexed images, the alpha channel is stored
-    //! separately (in this data structure, anyway).
-    Tiles alpha_tiles;
-    Tiles mask_tiles;		//!< The layer mask (optional)
-
-    //! Additional information about a layer mask.
-    struct {
-      Q_UINT32 opacity;
-      Q_UINT32 visible;
-      Q_UINT32 show_masked;
-      uchar red, green, blue;
-      Q_UINT32 tattoo;
-    } mask_channel;
-
-    bool active;		//!< Is this layer the active layer?
-    Q_UINT32 opacity;		//!< The opacity of the layer
-    Q_UINT32 visible;		//!< Is the layer visible?
-    Q_UINT32 linked;		//!< Is this layer linked (geometrically)
-    Q_UINT32 preserve_transparency; //!< Preserve alpha when drawing on layer?
-    Q_UINT32 apply_mask;	//!< Apply the layer mask?
-    Q_UINT32 edit_mask;		//!< Is the layer mask the being edited?
-    Q_UINT32 show_mask;		//!< Show the layer mask rather than the image?
-    Q_INT32 x_offset;		//!< x offset of the layer relative to the image
-    Q_INT32 y_offset;		//!< y offset of the layer relative to the image
-    Q_UINT32 mode;		//!< Combining mode of layer (LayerModeEffects)
-    Q_UINT32 tattoo;		//!< (unique identifier?)
-
-    //! As each tile is read from the file, it is buffered here.
-    uchar tile[TILE_WIDTH * TILE_HEIGHT * sizeof(QRgb)];
-
-    //! The data from tile buffer is copied to the Tile by this
-    //! method.  Depending on the type of the tile (RGB, Grayscale,
-    //! Indexed) and use (image or mask), the bytes in the buffer are
-    //! copied in different ways.
-    void (*assignBytes)( Layer& layer, uint i, uint j );
-
-    //! Construct a layer.
-    Layer ( void ) : name( 0 ) {}
-    //! Destruct the layer.
-    ~Layer ( void ) { if ( name != 0 ) delete[] name; }
-  };
-
-  /*!
-   * The in-memory representation of the XCF Image. It contains a few
-   * metadata items, but is mostly a container for the layer information.
-   */
-  struct XCFImage {
-    Q_UINT32 width;		//!< width of the XCF image
-    Q_UINT32 height;		//!< height of the XCF image
-    Q_INT32 type;		//!< type of the XCF image (GimpImageBaseType)
-
-    Q_UINT8 compression;	//!< tile compression method (CompressionType)
-    float x_resolution;		//!< x resolution in dots per inch
-    float y_resolution;		//!< y resolution in dots per inch
-    Q_INT32 tattoo;		//!< (unique identifier?)
-    Q_UINT32 unit;		//!< Units of The GIMP (inch, mm, pica, etc...)
-    Q_INT32 num_colors;		//!< number of colors in an indexed image
-    QValueVector< QRgb > palette; //!< indexed image color palette
-
-    int num_layers;		//!< number of layers
-    Layer layer;		//!< most recently read layer
-
-    bool initialized;		//!< Is the QImage initialized?
-    QImage image;		//!< final QImage
-
-    //! Simple constructor.
-    XCFImage ( void ) : initialized( false ) {}
-  };
-
-  //! The bottom-most layer is copied into the final QImage by this
-  //! routine.
-  typedef void (*PixelCopyOperation) ( Layer& layer, uint i, uint j, int k, int l,
-				       QImage& image, int m, int n );
-
-  //! Higher layers are merged into the the final QImage by this routine.
-  typedef void (*PixelMergeOperation) ( Layer& layer, uint i, uint j, int k, int l,
-					QImage& image, int m, int n );
-
-  //! In layer DISSOLVE mode, a random number is chosen to compare to a
-  //! pixel's alpha. If the alpha is greater than the random number, the
-  //! pixel is drawn. This table merely contains the random number seeds
-  //! for each ROW of an image. Therefore, the random numbers chosen
-  //! are consistent from run to run.
-
-  static int random_table[RANDOM_TABLE_SIZE];
-
-  //! This table provides the add_pixel saturation values (i.e. 250 + 250 = 255).
-
-  static int add_lut[256][256];
-
-  //! Layer mode static data.
-  typedef struct {
-    bool affect_alpha;		//!< Does this mode affect the source alpha?
-  } LayerModes;
-
-  //! Array of layer mode structures for the modes described by
-  //! LayerModeEffects.
-  static LayerModes layer_modes[];
-
-public:
-  /*!
-   * The constructor for the XCF image loader. This initializes the
-   * tables used in the layer merging routines.
-   */
-  XCFImageFormat ()
-  {
-    // From GIMP "paint_funcs.c" v1.2
-    srand( RANDOM_SEED );
-
-    for ( int i = 0; i < RANDOM_TABLE_SIZE; i++ )
-      random_table[i] = rand();
-
-    for ( int i = 0; i < RANDOM_TABLE_SIZE; i++ ) {
-      int tmp;
-      int swap = i + rand() % ( RANDOM_TABLE_SIZE - i );
-      tmp = random_table[i];
-      random_table[i] = random_table[swap];
-      random_table[swap] = tmp;
-    }
-
-    for ( int j = 0; j < 256; j++ ) {
-      for ( int k = 0; k < 256; k++ ) {
-	int tmp_sum = j + k;
-	if ( tmp_sum > 255 )
-	  tmp_sum = 255;
-	add_lut[j][k] = tmp_sum;
-      }
-    }
-  }
-
-  /*!
-   * The image loader makes no (direct) use of dynamic memory
-   * and the Qt infrastructure takes care of constructing and destructing
-   * the loader so there is not much to do here.
-   */
-  ~XCFImageFormat () {}
-
-  /*!
-   * You can query Qt about the types of image file formats it knows about
-   * via QImage::inputFormats or QImage::inputFormatList().
-   * This method returns "xcf".
-   */
-  QStringList keys () const {
-    return QStringList() << "XCF";
-  }
-
-  /*!
-   * This method installs the XCF reader on demand.
-   */
-  bool installIOHandler ( const QString& ) {
-    QImageIO::defineIOHandler( "XCF", "gimp xcf", 0,
-			       &XCFImageFormat::readXCF,
-#ifdef TMP_WRITE
-			       &XCFImageFormat::writeXCF );
-#else
-			       0 );
-#endif
-    return true;
-  }
-
-  static void readXCF ( QImageIO* image_io );
-#ifdef TMP_WRITE
-  static void writeXCF ( QImageIO* ) {}
-#endif
-private:
-  static void initializeImage ( XCFImage& xcf_image );
-  static void composeTiles ( XCFImage& xcf_image );
-  static bool loadImageProperties ( QDataStream& xcf_io, XCFImage& image );
-  static bool loadLayer ( QDataStream& xcf_io, XCFImage& xcf_image );
-  static bool loadLayerProperties ( QDataStream& xcf_io, Layer& layer );
-  static bool loadChannelProperties ( QDataStream& xcf_io, Layer& layer );
-  static bool loadHierarchy ( QDataStream& xcf_io, Layer& layer );
-  static bool loadMask ( QDataStream& xcf_io, Layer& layer );
-  static bool loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp );
-  static bool loadTileRLE ( QDataStream& xcf_io, uchar* tile, int size,
-			    int data_length, Q_INT32 bpp );
-  static bool loadProperty ( QDataStream& xcf_io, PropType& type,
-			     QByteArray& bytes );
-  static void setGrayPalette ( QImage& image );
-  static void setPalette ( XCFImage& xcf_image, QImage& image );
-  static void assignImageBytes ( Layer& layer, uint i, uint j );
-  static void assignMaskBytes ( Layer& layer, uint i, uint j );
-
-  static void copyLayerToImage ( XCFImage& xcf_image );
-  static void copyRGBToRGB ( Layer& layer, uint i, uint j, int k, int l,
-			     QImage& image, int m, int n );
-  static void copyGrayToGray ( Layer& layer, uint i, uint j, int k, int l,
-			       QImage& image, int m, int n );
-  static void copyGrayToRGB ( Layer& layer, uint i, uint j, int k, int l,
-			      QImage& image, int m, int n );
-  static void copyGrayAToRGB ( Layer& layer, uint i, uint j, int k, int l,
-			       QImage& image, int m, int n );
-  static void copyIndexedToIndexed ( Layer& layer, uint i, uint j, int k, int l,
-				     QImage& image, int m, int n );
-  static void copyIndexedAToIndexed ( Layer& layer, uint i, uint j, int k, int l,
-				      QImage& image, int m, int n );
-  static void copyIndexedAToRGB ( Layer& layer, uint i, uint j, int k, int l,
-				  QImage& image, int m, int n );
-
-  static void mergeLayerIntoImage ( XCFImage& xcf_image );
-  static void mergeRGBToRGB ( Layer& layer, uint i, uint j, int k, int l,
-			      QImage& image, int m, int n );
-  static void mergeGrayToGray ( Layer& layer, uint i, uint j, int k, int l,
-				QImage& image, int m, int n );
-  static void mergeGrayAToGray ( Layer& layer, uint i, uint j, int k, int l,
-				 QImage& image, int m, int n );
-  static void mergeGrayToRGB ( Layer& layer, uint i, uint j, int k, int l,
-			       QImage& image, int m, int n );
-  static void mergeGrayAToRGB ( Layer& layer, uint i, uint j, int k, int l,
-				QImage& image, int m, int n );
-  static void mergeIndexedToIndexed ( Layer& layer, uint i, uint j, int k, int l,
-				      QImage& image, int m, int n );
-  static void mergeIndexedAToIndexed ( Layer& layer, uint i, uint j, int k, int l,
-				       QImage& image, int m, int n );
-  static void mergeIndexedAToRGB ( Layer& layer, uint i, uint j, int k, int l,
-				   QImage& image, int m, int n );
-
-  static void dissolveRGBPixels ( QImage& image, int x, int y );
-  static void dissolveAlphaPixels ( QImage& image, int x, int y );
-};
-
 // Static global values
 
 int XCFImageFormat::random_table[RANDOM_TABLE_SIZE];
@@ -345,6 +61,336 @@ inline QRgb qRgba ( QRgb rgb, int a )
   return ( ( a & 0xff ) << 24 | ( rgb & RGB_MASK ) );
 }
 
+
+//////////////////////////////////////////////////////////////////////////////////
+// From GIMP "paint_funcs.c" v1.2
+
+/*!
+ * Multiply two color components. Really expects the arguments to be
+ * 8-bit quantities.
+ * \param a first minuend.
+ * \param b second minuend.
+ * \return product of arguments.
+ */
+inline int INT_MULT ( int a, int b )
+{
+  int c = a * b + 0x80;
+  return ( ( c >> 8 ) + c ) >> 8;
+}
+
+/*!
+ * Blend the two color components in the proportion alpha:
+ *
+ * result = alpha a + ( 1 - alpha b)
+ *
+ * \param a first component.
+ * \param b second component.
+ * \param alpha blend proportion.
+ * \return blended color components.
+ */
+
+inline int INT_BLEND ( int a, int b, int alpha )
+{
+  return INT_MULT( a - b, alpha ) + b;
+}
+
+// Actually from GLIB
+
+inline int MIN ( int a, int b )
+{
+  return ( a < b ? a : b );
+}
+
+inline int MAX ( int a, int b )
+{
+  return ( a > b ? a : b );
+}
+
+// From GIMP "gimpcolorspace.c" v1.2
+
+/*!
+ * Convert a color in RGB space to HSV space (Hue, Saturation, Value).
+ * \param red the red component (modified in place).
+ * \param green the green component (modified in place).
+ * \param blue the blue component (modified in place).
+ */
+void RGBTOHSV ( uchar& red, uchar& green, uchar& blue )
+{
+  int r, g, b;
+  double h, s, v;
+  int min, max;
+
+  h = 0.;
+
+  r = red;
+  g = green;
+  b = blue;
+
+  if ( r > g ) {
+    max = MAX( r, b );
+    min = MIN( g, b );
+  }
+  else {
+    max = MAX( g, b );
+    min = MIN( r, b );
+  }
+
+  v = max;
+
+  if ( max != 0 )
+    s = ( ( max - min ) * 255 ) / (double)max;
+  else
+    s = 0;
+
+  if ( s == 0 )
+    h = 0;
+  else {
+    int delta = max - min;
+    if ( r == max )
+      h = ( g - b ) / (double)delta;
+    else if ( g == max )
+      h = 2 + ( b - r ) / (double)delta;
+    else if ( b == max )
+      h = 4 + ( r - g ) / (double)delta;
+    h *= 42.5;
+
+    if ( h < 0 )
+      h += 255;
+    if ( h > 255 )
+      h -= 255;
+  }
+
+  red   = (uchar)h;
+  green = (uchar)s;
+  blue  = (uchar)v;
+}
+
+/*!
+ * Convert a color in HSV space to RGB space.
+ * \param hue the hue component (modified in place).
+ * \param saturation the saturation component (modified in place).
+ * \param value the value component (modified in place).
+ */
+void HSVTORGB ( uchar& hue, uchar& saturation, uchar& value )
+{
+  if ( saturation == 0 ) {
+    hue        = value;
+    saturation = value;
+    value      = value;
+  }
+  else {
+    double h = hue * 6. / 255.;
+    double s = saturation / 255.;
+    double v = value / 255.;
+
+    double f = h - (int)h;
+    double p = v * ( 1. - s );
+    double q = v * ( 1. - ( s * f ) );
+    double t = v * ( 1. - ( s * ( 1. - f ) ) );
+
+    // Worth a note here that gcc 2.96 will generate different results
+    // depending on optimization mode on i386.
+
+    switch ((int)h) {
+    case 0:
+      hue        = (uchar)( v * 255 );
+      saturation = (uchar)( t * 255 );
+      value      = (uchar)( p * 255 );
+      break;
+    case 1:
+      hue        = (uchar)( q * 255 );
+      saturation = (uchar)( v * 255 );
+      value      = (uchar)( p * 255 );
+      break;
+    case 2:
+      hue        = (uchar)( p * 255 );
+      saturation = (uchar)( v * 255 );
+      value      = (uchar)( t * 255 );
+      break;
+    case 3:
+      hue        = (uchar)( p * 255 );
+      saturation = (uchar)( q * 255 );
+      value      = (uchar)( v * 255 );
+      break;
+    case 4:
+      hue        = (uchar)( t * 255 );
+      saturation = (uchar)( p * 255 );
+      value      = (uchar)( v * 255 );
+      break;
+    case 5:
+      hue        = (uchar)( v * 255 );
+      saturation = (uchar)( p * 255 );
+      value      = (uchar)( q * 255 );
+    }
+  }
+}
+
+/*!
+ * Convert a color in RGB space to HLS space (Hue, Lightness, Saturation).
+ * \param red the red component (modified in place).
+ * \param green the green component (modified in place).
+ * \param blue the blue component (modified in place).
+ */
+void RGBTOHLS ( uchar& red, uchar& green, uchar& blue )
+{
+  int r = red;
+  int g = green;
+  int b = blue;
+
+  int min, max;
+
+  if ( r > g ) {
+    max = MAX( r, b );
+    min = MIN( g, b );
+  }
+  else {
+    max = MAX( g, b );
+    min = MIN( r, b );
+  }
+
+  double h;
+  double l = ( max + min ) / 2.;
+  double s;
+
+  if ( max == min ) {
+    s = 0.;
+    h = 0.;
+  }
+  else {
+    int delta = max - min;
+
+    if ( l < 128 )
+      s = 255 * (double)delta / (double)( max + min );
+    else
+      s = 255 * (double)delta / (double)( 511 - max - min );
+
+    if ( r == max )
+      h = ( g - b ) / (double)delta;
+    else if ( g == max )
+      h = 2 + ( b - r ) / (double)delta;
+    else
+      h = 4 + ( r - g ) / (double)delta;
+
+    h *= 42.5;
+
+    if ( h < 0 )
+      h += 255;
+    else if ( h > 255 )
+      h -= 255;
+  }
+
+  red   = (uchar)h;
+  green = (uchar)l;
+  blue  = (uchar)s;
+}
+
+/*!
+ * Implement the HLS "double hex-cone".
+ * \param n1 lightness fraction (?)
+ * \param n2 saturation fraction (?)
+ * \param hue hue "angle".
+ * \return HLS value.
+ */
+int HLSVALUE ( double n1, double n2, double hue )
+{
+  double value;
+
+  if ( hue > 255 )
+    hue -= 255;
+  else if ( hue < 0 )
+    hue += 255;
+
+  if ( hue < 42.5 )
+    value = n1 + ( n2 - n1 ) * ( hue / 42.5 );
+  else if ( hue < 127.5 )
+    value = n2;
+  else if ( hue < 170 )
+    value = n1 + ( n2 - n1 ) * ( ( 170 - hue ) / 42.5 );
+  else
+    value = n1;
+
+  return (int)( value * 255 );
+}
+
+/*!
+ * Convert a color in HLS space to RGB space.
+ * \param hue the hue component (modified in place).
+ * \param lightness the lightness component (modified in place).
+ * \param saturation the saturation component (modified in place).
+ */
+void HLSTORGB ( uchar& hue, uchar& lightness, uchar& saturation )
+{
+  double h = hue;
+  double l = lightness;
+  double s = saturation;
+
+  if ( s == 0 ) {
+    hue        = (uchar)l;
+    lightness  = (uchar)l;
+    saturation = (uchar)l;
+  }
+  else {
+    double m1, m2;
+
+    if ( l < 128 )
+      m2 = ( l * ( 255 + s ) ) / 65025.;
+    else
+      m2 = ( l + s - ( l * s ) / 255. ) / 255.;
+
+    m1 = ( l / 127.5 ) - m2;
+
+    hue        = HLSVALUE( m1, m2, h + 85 );
+    lightness  = HLSVALUE( m1, m2, h );
+    saturation = HLSVALUE( m1, m2, h - 85 );
+  }
+}
+//////////////////////////////////////////////////////////////////////////////////
+
+
+XCFImageFormat::XCFImageFormat() {
+    // From GIMP "paint_funcs.c" v1.2
+    srand( RANDOM_SEED );
+
+    for ( int i = 0; i < RANDOM_TABLE_SIZE; i++ )
+      random_table[i] = rand();
+
+    for ( int i = 0; i < RANDOM_TABLE_SIZE; i++ ) {
+      int tmp;
+      int swap = i + rand() % ( RANDOM_TABLE_SIZE - i );
+      tmp = random_table[i];
+      random_table[i] = random_table[swap];
+      random_table[swap] = tmp;
+    }
+
+    for ( int j = 0; j < 256; j++ ) {
+      for ( int k = 0; k < 256; k++ ) {
+	int tmp_sum = j + k;
+	if ( tmp_sum > 255 )
+	  tmp_sum = 255;
+	add_lut[j][k] = tmp_sum;
+      }
+    }
+  }
+
+
+bool XCFImageFormat::installIOHandler ( const QString& ) {
+    QImageIO::defineIOHandler( "XCF", "gimp xcf", 0,
+			       &XCFImageFormat::readXCF,
+#ifdef TMP_WRITE
+			       &XCFImageFormat::writeXCF );
+#else
+			       0 );
+#endif
+    return true;
+}
+
+
+void XCFImageFormat::registerFormat() {
+	QImageIO::defineIOHandler( "XCF","^gimp xcf",
+		0,XCFImageFormat::readXCF,0L);
+}
+
+
 /*!
  * The Qt QImageIO architecture invokes this routine to read the image.
  * The file (or other data stream) is already open and the
@@ -352,9 +398,9 @@ inline QRgb qRgba ( QRgb rgb, int a )
  * is positioned at its beginning).
  *
  * The XCF file is binary and is stored in big endian format. The
- * QDataStream class is used to read the file. Even though the XCF file
- * was not written with QDataStream, there is still a good match. At least
- * in version 001 of XCF and version 4 of QDataStream. Any other combination
+ * SafeDataStream class is used to read the file. Even though the XCF file
+ * was not written with SafeDataStream, there is still a good match. At least
+ * in version 001 of XCF and version 4 of SafeDataStream. Any other combination
  * is suspect.
  *
  * Any failures while reading the XCF image are reported by the
@@ -366,23 +412,23 @@ void XCFImageFormat::readXCF ( QImageIO* image_io )
 {
   XCFImage xcf_image;
 
-  // The XCF data is stored in big endian format, which QDataStream handles
+  // The XCF data is stored in big endian format, which SafeDataStream handles
   // very well.
 
-  QDataStream xcf_io( image_io->ioDevice() );
+  SafeDataStream xcf_io( image_io->ioDevice() );
 
   char tag[14];
   xcf_io.readRawBytes( tag, sizeof(tag) );
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on header tag" );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on header tag"<< endl;
     return;
   }
 
   xcf_io >> xcf_image.width >> xcf_image.height >> xcf_image.type;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on image info" );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on image info" << endl;
     return;
   }
 
@@ -401,8 +447,8 @@ void XCFImageFormat::readXCF ( QImageIO* image_io )
 
     xcf_io >> layer_offset;
 
-    if ( xcf_io.device()->status() != IO_Ok ) {
-      qDebug( "XCF: read failure on layer offsets" );
+    if ( xcf_io.failed() ) {
+      kdDebug() << "XCF: read failure on layer offsets"<< endl;
       return;
     }
 
@@ -411,10 +457,10 @@ void XCFImageFormat::readXCF ( QImageIO* image_io )
     layer_offsets.push( layer_offset );
   }
 
-  xcf_image.num_layers = layer_offsets.count();
+  xcf_image.num_layers = layer_offsets.size();
 
-  if ( layer_offsets.count() == 0 ) {
-    qDebug( "XCF: no layers!" );
+  if ( layer_offsets.size() == 0 ) {
+    kdDebug() << "XCF: no layers!"<< endl;
     return;
   }
 
@@ -429,7 +475,7 @@ void XCFImageFormat::readXCF ( QImageIO* image_io )
   }
 
   if ( !xcf_image.initialized ) {
-    qDebug( "XCF: no visible layers!" );
+    kdDebug() << "XCF: no visible layers!"<< endl;
     return;
   }
 
@@ -504,7 +550,7 @@ void XCFImageFormat::initializeImage ( XCFImage& xcf_image )
   case INDEXED_GIMAGE:
     // As noted in the table above, there are quite a few combinations
     // which are possible with indexed images, depending on the
-    // presense of transparency (note: not translucency, which is not
+    // presence of transparency (note: not translucency, which is not
     // supported by The GIMP for indexed images) and the number of
     // individual colors.
 
@@ -703,7 +749,7 @@ void XCFImageFormat::setPalette ( XCFImage& xcf_image, QImage& image )
  * \param xcf_image XCF image data.
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadImageProperties ( QDataStream& xcf_io,
+bool XCFImageFormat::loadImageProperties ( SafeDataStream& xcf_io,
 					   XCFImage& xcf_image )
 {
   while ( true ) {
@@ -711,7 +757,7 @@ bool XCFImageFormat::loadImageProperties ( QDataStream& xcf_io,
     QByteArray bytes;
 
     if ( !loadProperty( xcf_io, type, bytes ) ) {
-      qDebug( "XCF: error loading global image properties" );
+      kdDebug() << "XCF: error loading global image properties"<< endl;
       return false;
     }
 
@@ -725,10 +771,10 @@ bool XCFImageFormat::loadImageProperties ( QDataStream& xcf_io,
       property >> xcf_image.compression;
       break;
 
-     case PROP_GUIDES:
-       // This property is ignored.
-       break;
-  	
+    case PROP_GUIDES:
+      // This property is ignored.
+      break;
+	  
     case PROP_RESOLUTION:
       property >> xcf_image.x_resolution >> xcf_image.y_resolution;
       break;
@@ -781,8 +827,7 @@ bool XCFImageFormat::loadImageProperties ( QDataStream& xcf_io,
       break;
 
     default:
-      //qDebug( "XCF: unimplemented image property %d, size %d", type, bytes.size() );
-      ;
+      kdDebug() << "XCF: unimplemented image property d, size d"<< type<<" "<< bytes.size()<< endl;
     }
   }
 }
@@ -795,7 +840,7 @@ bool XCFImageFormat::loadImageProperties ( QDataStream& xcf_io,
  * (if the image is indexed).
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadLayer ( QDataStream& xcf_io, XCFImage& xcf_image )
+bool XCFImageFormat::loadLayer ( SafeDataStream& xcf_io, XCFImage& xcf_image )
 {
   Layer& layer( xcf_image.layer );
 
@@ -803,8 +848,8 @@ bool XCFImageFormat::loadLayer ( QDataStream& xcf_io, XCFImage& xcf_image )
 
   xcf_io >> layer.width >> layer.height >> layer.type >> layer.name;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on layer" );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on layer"<< endl;
     return false;
   }
 
@@ -825,8 +870,8 @@ bool XCFImageFormat::loadLayer ( QDataStream& xcf_io, XCFImage& xcf_image )
 
   xcf_io >> layer.hierarchy_offset >> layer.mask_offset;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on layer image offsets" );
+  if ( xcf_io.failed() ) {
+    kdDebug() <<"XCF: read failure on layer image offsets" << endl;
     return false;
   }
 
@@ -875,14 +920,14 @@ bool XCFImageFormat::loadLayer ( QDataStream& xcf_io, XCFImage& xcf_image )
  * \param layer layer to collect the properties.
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadLayerProperties ( QDataStream& xcf_io, Layer& layer )
+bool XCFImageFormat::loadLayerProperties ( SafeDataStream& xcf_io, Layer& layer )
 {
   while ( true ) {
     PropType type;
     QByteArray bytes;
 
     if ( !loadProperty( xcf_io, type, bytes ) ) {
-      qDebug( "XCF: error loading layer properties" );
+      kdDebug() <<  "XCF: error loading layer properties"<< endl;
       return false;
     }
 
@@ -937,7 +982,7 @@ bool XCFImageFormat::loadLayerProperties ( QDataStream& xcf_io, Layer& layer )
       break;
 
     default:
-      qDebug( "XCF: unimplemented layer property %d, size %d", type, bytes.size() );
+      kdDebug() << "XCF: unimplemented layer property d, size d"<< type<<" "<< bytes.size()<< endl;
     }
   }
 }
@@ -949,14 +994,14 @@ bool XCFImageFormat::loadLayerProperties ( QDataStream& xcf_io, Layer& layer )
  * \param layer layer containing the mask channel to collect the properties.
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadChannelProperties ( QDataStream& xcf_io, Layer& layer )
+bool XCFImageFormat::loadChannelProperties ( SafeDataStream& xcf_io, Layer& layer )
 {
   while ( true ) {
     PropType type;
     QByteArray bytes;
 
     if ( !loadProperty( xcf_io, type, bytes ) ) {
-      qDebug( "XCF: error loading channel properties" );
+      kdDebug() << "XCF: error loading channel properties" << endl;
       return false;
     }
 
@@ -988,7 +1033,7 @@ bool XCFImageFormat::loadChannelProperties ( QDataStream& xcf_io, Layer& layer )
       break;
 
     default:
-      qDebug( "XCF: unimplemented channel property %d, size %d", type, bytes.size() );
+      kdDebug() << "XCF: unimplemented channel property d, size d"<< type<<" "<< bytes.size()<< endl;
     }
   }
 }
@@ -1001,7 +1046,7 @@ bool XCFImageFormat::loadChannelProperties ( QDataStream& xcf_io, Layer& layer )
  * \param layer the layer to collect the image.
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadHierarchy ( QDataStream& xcf_io, Layer& layer )
+bool XCFImageFormat::loadHierarchy ( SafeDataStream& xcf_io, Layer& layer )
 {
   Q_INT32 width;
   Q_INT32 height;
@@ -1010,8 +1055,8 @@ bool XCFImageFormat::loadHierarchy ( QDataStream& xcf_io, Layer& layer )
 
   xcf_io >> width >> height >> bpp >> offset;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on layer %s image header", layer.name );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on layer s image header "<< layer.name << endl;
     return false;
   }
 
@@ -1023,13 +1068,13 @@ bool XCFImageFormat::loadHierarchy ( QDataStream& xcf_io, Layer& layer )
   do {
     xcf_io >> junk;
 
-    if ( xcf_io.device()->status() != IO_Ok ) {
-      qDebug( "XCF: read failure on layer %s level offsets", layer.name );
+    if ( xcf_io.failed() ) {
+      kdDebug() << "XCF: read failure on layer s level offsets "<< layer.name << endl;
       return false;
     }
   } while ( junk != 0 );
 
-  int saved_pos = xcf_io.device()->at();
+  QIODevice::Offset saved_pos = xcf_io.device()->at();
 
   xcf_io.device()->at( offset );
 
@@ -1048,7 +1093,7 @@ bool XCFImageFormat::loadHierarchy ( QDataStream& xcf_io, Layer& layer )
  * \return true if there were no I/O errors.
  * \sa loadTileRLE().
  */
-bool XCFImageFormat::loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp )
+bool XCFImageFormat::loadLevel ( SafeDataStream& xcf_io, Layer& layer, Q_INT32 bpp )
 {
   Q_INT32 width;
   Q_INT32 height;
@@ -1056,8 +1101,8 @@ bool XCFImageFormat::loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp
 
   xcf_io >> width >> height >> offset;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on layer %s level info", layer.name );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on layer %s level info "<< layer.name << endl;
     return false;
   }
 
@@ -1067,19 +1112,18 @@ bool XCFImageFormat::loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp
     for ( uint i = 0; i < layer.ncols; i++ ) {
 
       if ( offset == 0 ) {
-	qDebug( "XCF: incorrect number of tiles in layer %s", layer.name );
+	kdDebug() << "XCF: incorrect number of tiles in layer " << layer.name << endl;
 	return false;
       }
 
-      int saved_pos = xcf_io.device()->at();
+      QIODevice::Offset saved_pos = xcf_io.device()->at();
 
       Q_UINT32 offset2;
 
       xcf_io >> offset2;
 
-      if ( xcf_io.device()->status() != IO_Ok ) {
-	qDebug( "XCF: read failure on layer %s level offset look-ahead",
-		layer.name );
+      if ( xcf_io.failed() ) {
+	kdDebug() << "XCF: read failure on layer s level offset look-ahead " << layer.name << endl;
 	return false;
       }
 
@@ -1105,8 +1149,8 @@ bool XCFImageFormat::loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp
       
       xcf_io >> offset;
 
-      if ( xcf_io.device()->status() != IO_Ok ) {
-	qDebug( "XCF: read failure on layer %s level offset", layer.name );
+      if ( xcf_io.failed() ) {
+	kdDebug() << "XCF: read failure on layer s level offset "<< layer.name<< endl;
 	return false;
       }
     }
@@ -1121,7 +1165,7 @@ bool XCFImageFormat::loadLevel ( QDataStream& xcf_io, Layer& layer, Q_INT32 bpp
  * \param layer the layer to collect the mask image.
  * \return true if there were no I/O errors.
  */
-bool XCFImageFormat::loadMask ( QDataStream& xcf_io, Layer& layer )
+bool XCFImageFormat::loadMask ( SafeDataStream& xcf_io, Layer& layer )
 {
   Q_INT32 width;
   Q_INT32 height;
@@ -1129,8 +1173,8 @@ bool XCFImageFormat::loadMask ( QDataStream& xcf_io, Layer& layer )
 
   xcf_io >> width >> height >> name;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on mask info" );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on mask info"<< endl;
     return false;
   }
 
@@ -1142,8 +1186,8 @@ bool XCFImageFormat::loadMask ( QDataStream& xcf_io, Layer& layer )
 
   xcf_io >> hierarchy_offset;
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on mask image offset" );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on mask image offset" << endl;
     return false;
   }
 
@@ -1179,7 +1223,7 @@ bool XCFImageFormat::loadMask ( QDataStream& xcf_io, Layer& layer )
  * \return true if there were no I/O errors and no obvious corruption of
  * the RLE data.
  */
-bool XCFImageFormat::loadTileRLE ( QDataStream& xcf_io, uchar* tile, int image_size,
+bool XCFImageFormat::loadTileRLE ( SafeDataStream& xcf_io, uchar* tile, int image_size,
 				   int data_length, Q_INT32 bpp )
 {
   uchar* data;
@@ -1190,15 +1234,15 @@ bool XCFImageFormat::loadTileRLE ( QDataStream& xcf_io, uchar* tile, int image_s
 
   xcfdata = xcfodata = new uchar[data_length];
 
-  xcf_io.readRawBytes( (char*)xcfdata, data_length  );
+  int read_length=xcf_io.device()->readBlock( (char*)xcfdata, data_length  );
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
+  if ( read_length<=0 ) {
     delete[] xcfodata;
-    qDebug( "XCF: read failure on tile" );
+    kdDebug() <<  "XCF: read failure on tile" << endl;
     return false;
   }
 
-  xcfdatalimit = &xcfodata[data_length-1];
+  xcfdatalimit = &xcfodata[read_length-1];
 
   for ( int i = 0; i < bpp; ++i ) {
 
@@ -1275,7 +1319,7 @@ bool XCFImageFormat::loadTileRLE ( QDataStream& xcf_io, uchar* tile, int image_s
 
  bogus_rle:
 
-  qDebug( "The run length encoding could not be decoded properly" );
+  kdDebug() << "The run length encoding could not be decoded properly"<< endl;
   delete[] xcfodata;
   return false;
 }
@@ -1366,15 +1410,15 @@ void XCFImageFormat::assignMaskBytes ( Layer& layer, uint i, uint j )
  * \param type returns with the property type.
  * \param bytes returns with the property data.
  * \return true if there were no IO errors.  */
-bool XCFImageFormat::loadProperty ( QDataStream& xcf_io, PropType& type,
+bool XCFImageFormat::loadProperty ( SafeDataStream& xcf_io, PropType& type,
 				    QByteArray& bytes )
 {
   Q_UINT32 tmp;
   xcf_io >> tmp;
   type=static_cast<PropType>(tmp);
- 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on property type" );
+
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on property type"<< endl;
     return false;
   }
 
@@ -1388,8 +1432,8 @@ bool XCFImageFormat::loadProperty ( QDataStream& xcf_io, PropType& type,
   if ( type == PROP_COLORMAP ) {
     xcf_io >> size;
 
-    if ( xcf_io.device()->status() != IO_Ok ) {
-      qDebug( "XCF: read failure on property %d size", type );
+    if ( xcf_io.failed() ) {
+      kdDebug() << "XCF: read failure on property d size "<< type  << endl;
       return false;
     }
 
@@ -1408,16 +1452,16 @@ bool XCFImageFormat::loadProperty ( QDataStream& xcf_io, PropType& type,
 
     xcf_io >> size >> factor >> digits;
 
-    if ( xcf_io.device()->status() != IO_Ok ) {
-      qDebug( "XCF: read failure on property %d", type );
+    if ( xcf_io.failed() ) {
+      kdDebug() <<  "XCF: read failure on property d "<< type << endl;
       return false;
     }
 
     for ( int i = 0; i < 5; i++ ) {
       xcf_io >> unit_strings;
 
-      if ( xcf_io.device()->status() != IO_Ok ) {
-	qDebug( "XCF: read failure on property %d", type );
+      if ( xcf_io.failed() ) {
+	kdDebug() << "XCF: read failure on property d "<< type << endl;
 	return false;
       }
 
@@ -1430,8 +1474,8 @@ bool XCFImageFormat::loadProperty ( QDataStream& xcf_io, PropType& type,
   else
     xcf_io.readBytes( data, size );
 
-  if ( xcf_io.device()->status() != IO_Ok ) {
-    qDebug( "XCF: read failure on property %d data, size %d", type, size );
+  if ( xcf_io.failed() ) {
+    kdDebug() << "XCF: read failure on property d data, size d"<< type <<" "<< size << endl;
     return false;
   }
 
@@ -2334,11 +2378,4 @@ void XCFImageFormat::dissolveAlphaPixels ( QImage& image, int x, int y )
   }
 }
 
-// For QT2 image handling
-void qimageio_readXCF(QImageIO *io)
-{
-    XCFImageFormat handler;
-    handler.readXCF(io);
-}
-
-//Q_EXPORT_PLUGIN( XCFImageFormat )
+Q_EXPORT_PLUGIN( XCFImageFormat )
